#include <linux/of_device.h>
#include <linux/mm.h>

#include <asm/io.h>

#include <esp_accelerator.h>
#include <esp.h>

#include "mlp3layers.h"

#define DRV_NAME	"mlp3layers"

/* <<--regs-->> */
#define MLP3LAYERS_NBURSTS_REG 0x40

struct mlp3layers_device {
	struct esp_device esp;
};

static struct esp_driver mlp3layers_driver;

static struct of_device_id mlp3layers_device_ids[] = {
	{
		.name = "SLD_MLP3LAYERS",
	},
	{
		.name = "eb_194",
	},
	{
		.compatible = "sld,mlp3layers",
	},
	{ },
};

static int mlp3layers_devs;

static inline struct mlp3layers_device *to_mlp3layers(struct esp_device *esp)
{
	return container_of(esp, struct mlp3layers_device, esp);
}

static void mlp3layers_prep_xfer(struct esp_device *esp, void *arg)
{
	struct mlp3layers_access *a = arg;

	/* <<--regs-config-->> */
	iowrite32be(a->nbursts, esp->iomem + MLP3LAYERS_NBURSTS_REG);
	iowrite32be(a->src_offset, esp->iomem + SRC_OFFSET_REG);
	iowrite32be(a->dst_offset, esp->iomem + DST_OFFSET_REG);

}

static bool mlp3layers_xfer_input_ok(struct esp_device *esp, void *arg)
{
	/* struct mlp3layers_device *mlp3layers = to_mlp3layers(esp); */
	/* struct mlp3layers_access *a = arg; */

	return true;
}

static int mlp3layers_probe(struct platform_device *pdev)
{
	struct mlp3layers_device *mlp3layers;
	struct esp_device *esp;
	int rc;

	mlp3layers = kzalloc(sizeof(*mlp3layers), GFP_KERNEL);
	if (mlp3layers == NULL)
		return -ENOMEM;
	esp = &mlp3layers->esp;
	esp->module = THIS_MODULE;
	esp->number = mlp3layers_devs;
	esp->driver = &mlp3layers_driver;
	rc = esp_device_register(esp, pdev);
	if (rc)
		goto err;

	mlp3layers_devs++;
	return 0;
 err:
	kfree(mlp3layers);
	return rc;
}

static int __exit mlp3layers_remove(struct platform_device *pdev)
{
	struct esp_device *esp = platform_get_drvdata(pdev);
	struct mlp3layers_device *mlp3layers = to_mlp3layers(esp);

	esp_device_unregister(esp);
	kfree(mlp3layers);
	return 0;
}

static struct esp_driver mlp3layers_driver = {
	.plat = {
		.probe		= mlp3layers_probe,
		.remove		= mlp3layers_remove,
		.driver		= {
			.name = DRV_NAME,
			.owner = THIS_MODULE,
			.of_match_table = mlp3layers_device_ids,
		},
	},
	.xfer_input_ok	= mlp3layers_xfer_input_ok,
	.prep_xfer	= mlp3layers_prep_xfer,
	.ioctl_cm	= MLP3LAYERS_IOC_ACCESS,
	.arg_size	= sizeof(struct mlp3layers_access),
};

static int __init mlp3layers_init(void)
{
	return esp_driver_register(&mlp3layers_driver);
}

static void __exit mlp3layers_exit(void)
{
	esp_driver_unregister(&mlp3layers_driver);
}

module_init(mlp3layers_init)
module_exit(mlp3layers_exit)

MODULE_DEVICE_TABLE(of, mlp3layers_device_ids);

MODULE_AUTHOR("Emilio G. Cota <cota@braap.org>");
MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("mlp3layers driver");
